(defun C:WMF_World ( / MyScreenSize ScreenX ScreenY MyAspectRatio ViewHeight HalfView)
; K.J. McRae - Winnipeg Manitoba - July 2005
; Minor cleanup and commenting for City of Montreal - KJM - Sept 2015

; Write ESRI world file for use geo-referencing Windows Meta File Vector linework
; in a SWMM Model
; https://www.openswmm.org/Topic/4532/importing-data-from-autocad
; 
; Purpose:
;
; Exports an ESRI world coordinate file from the current AutoCAD viewport, to aid in
; geo-referencing background linework as Windows meta-files in SWMM Ver 5 Models.
; The routine will also draw a polyline around the viewport limits, which you can
; select and export the polyline along with your background map linework if desired.
;
; The routine will output a simplified ESRI world file, with parameters:
;    Line 1: A: pixel size in the x-direction in map units/pixel 
;    Line 2: D: rotation about y-axis <not implemented>
;    Line 3: B: rotation about x-axis <not implemented>
;    Line 4: E: pixel size in the y-direction in map units, almost always negative 
;    Line 5: C: x-coordinate of the center of the upper left pixel 
;    Line 6: F: y-coordinate of the center of the upper left pixel
; 
; For UTM coordinates:
;    D and B are usually 0, since the image pixels are usually made to aligned with the UTM grid 
;    C is the Easting UTM coordinate 
;    F is the Northing UTM coordinate 
;    Units are always metres per pixel
;
; The resultant file will be save with the name 'DrawingName.mfw'
; 
; Instructions for use:
; 
;  1.Zoom and pan in AutoCAD to get the best view for your background map.  It is best
;    to make the AutoCAD viewport windows approximately the same aspect ratio
;    as your SWMM5 or EPANET map window.
;
;  2.Run the included lisp routine from the AutoCAD command line.  Type 'wmf_world.
; 
;  3.Don't change the viewport limits, zoom, pan, etc!
;
;  4.Use AutoCAD commands EXPORT or WMFOUT to write desired linework to the .wmf file
;    (include boundary polyline if you like)
;
;  5.Open SWMM5 and choose menu items \view\backdrop\load
;
;  6.Select both the Windows Metafile.wmf and the ESRI world coordinates file.mfw
;
;  7.Done. Compare some coordinates between Acad and SWMM5/EPANET.

(setq ShowPrec 3)	; Decimal precision for numbers shown on screen
(setq OutputPrec 6)	; Decimal precision for output

(setq OldCmd (getvar "cmdecho"))
(setvar "cmdecho" 0)

; Screen size in pixels
(setq MyScreenSize (getvar "screensize"))	
(setq ScreenX (float (car MyScreenSize)))
(setq ScreenY (float (cadr MyScreenSize)))

(setq MyAspectRatio (/ ScreenX ScreenY))

(prompt "\nAcad viewport width = ")(princ (rtos ScreenX 2 ShowPrec))
(prompt "  height = ")(princ (rtos ScreenY 2 ShowPrec))
(prompt "  Aspect Ratio = ")(princ (rtos MyAspectRatio 2 ShowPrec))(princ)

; Height of the view in the current viewport (drawing units)
(setq ViewHeight (getvar "viewsize"))	

;Center of view in the current viewport (drawing units)
(setq MyCentreCoord (getvar "viewctr"))	
(setq CX (car MyCentreCoord))
(setq CY (cadr MyCentreCoord))

(setq OffsetX (* 0.5 ViewHeight MyAspectRatio))
(setq OffsetY (* 0.5 ViewHeight))

; Corner of viewport / screen (in drawing units)
(setq Xmin (- CX offsetX))
(setq Xmax (+ CX offsetX))

(setq Ymin (- CY offsetY))
(setq Ymax (+ CY offsetY))

; Display coordinates
(prompt "\nXmin = ")(princ (rtos Xmin 2 ShowPrec))(prompt "  Xmax = ")(princ (rtos Xmax 2 ShowPrec))(princ)
(prompt "\nYmin = ")(princ (rtos Ymin 2 ShowPrec))(prompt "  Ymax = ")(princ (rtos Ymax 2 ShowPrec))(princ)

; draw a polyline boundary around the viewport limits
(command "PLINE"
		(list Xmin Ymin 0.0)
		(list Xmax Ymin 0.0)
		(list Xmax Ymax 0.0)
		(list Xmin Ymax 0.0)
	"C") ; finish drawing polyline

; Size of Pixel
(setq XPixelSize (/ (- Xmax Xmin) ScreenX))	; positive
(setq YPixelSize (/ (- Ymin Ymax) ScreenY))	; typically negative (i.e. screen coordinates)
(prompt "\nX pixel size = ")(princ (rtos XPixelSize 2 ShowPrec))
(prompt "   Y pixel size = ")(princ (rtos YPixelSize 2 ShowPrec))(princ)

; Centre of Upper Left X and Y pixel (*** not upper left edge edge of pixel)
(setq XUL_Ctr (+ Xmin (* 0.5 XPixelSize)))	; half an x-pixel right of Xmin
(setq YUL_Ctr (+ Ymax (* 0.5 YPixelSize)))	; half a y-pixel down from Ymax (YPixelSize is negative)


; ----------------------------------------- 
; write ESRI world file (rename as needed later)
; use default extension .mfw for Windows Metafile (rename to .jfw for .jpeg world file
; or .tfw for .tiff world file)

(setq MyFullDwgName (getvar "DWGNAME"))
; drawing name without '.dwg' extension
(setq MyDwgNameOnly (substr MyFullDwgName 1 (- (strlen MyFullDwgName) 4)))

(setq Myfilename MyDwgNameOnly)
(setq Myfilename (getfiled "Save World File As" Myfilename "mfw" 1))

(setq file (open Myfilename "w"))

; Line 1: Pixel size in the x-direction (drawing units/pixel)
(princ (rtos XPixelSize 2 OutputPrec) file)
(princ "\n" file)

; Line 2: Rotation about y-axis (not used)
(princ (rtos 0.0 2 OutputPrec) file)
(princ "\n" file)

; Line 3: Rotation about x-axis (not used)
(princ (rtos 0.0 2 OutputPrec) file)
(princ "\n" file)

; Line 4: Pixel size in the y-direction (drawing units/pixel), almost always negative
(princ (rtos YPixelSize 2 OutputPrec) file)
(princ "\n" file)

; Line 5: X coordinate of centre of upper left pixel (screen x=0 y=0)
(princ (rtos XUL_CTR 2 OutputPrec) file)
(princ "\n" file)

; Line 6: Y coordinate of centre of upper left pixel
(princ (rtos YUL_CTR 2 OutputPrec) file)
(princ "\n" file)

(close file)

; Clean up
(setvar "cmdecho" OldCmd)

(prompt "\nCompleted.")
(princ)
) ; end function

(prompt "\nWMF_World - write ESRI world coordinates file for Windows Metafile export.")
(princ)
